<?php

/**
 * REST API Handler for App For Delivery Drivers
 *
 * @link       https://powerfulwp.com/
 * @since      1.0.0
 *
 * @package    App_For_Delivery_Drivers
 * @subpackage App_For_Delivery_Drivers/includes
 */

/**
 * REST API Handler for App For Delivery Drivers.
 *
 * This class defines all code necessary to run the REST API endpoints
 * for managing driver locations within the App For Delivery Drivers plugin.
 *
 * @since      1.0.0
 * @package    App_For_Delivery_Drivers
 * @subpackage App_For_Delivery_Drivers/includes
 * @author     powerfulwp <apowerfulwp@gmail.com>
 */
class App_For_Delivery_Drivers_API {

    /**
     * Constructor to initialize hooks.
     */
    public function __construct() {
        add_action( 'rest_api_init', array( $this, 'register_routes' ) );
    }

    /**
     * Registers the custom REST API routes.
     */
    public function register_routes() {
        register_rest_route( 'app-for-delivery-drivers/v1', '/set_driver_location', array(
            array(
                'methods'             => 'POST',
                'callback'            => array( $this, 'set_driver_location' ),
                'permission_callback' => array( $this, 'permissions_check' ),
                'args'                => array(
                    'driver_email' => array(
                        'required'          => true,
                        'type'              => 'string',
                        'description'       => 'Email address of the driver.',
                        'sanitize_callback' => 'sanitize_email',
                        'validate_callback' => array( $this, 'validate_email' ),
                    ),
                    'latitude' => array(
                        'required'          => true,
                        'type'              => 'number',
                        'description'       => 'Latitude of the driver\'s current location.',
                          'sanitize_callback' => array( $this, 'sanitize_float' ),
                        'validate_callback' => array( $this, 'validate_latitude' ),
                    ),
                    'longitude' => array(
                        'required'          => true,
                        'type'              => 'number',
                        'description'       => 'Longitude of the driver\'s current location.',
                          'sanitize_callback' => array( $this, 'sanitize_float' ),
                        'validate_callback' => array( $this, 'validate_longitude' ),
                    ),
                    'speed' => array(
                        'required'          => false,
                        'type'              => 'number',
                        'description'       => 'Current speed of the driver.',
                        'sanitize_callback' => array( $this, 'sanitize_float' ),
                    ),
                ),
            ),
        ) );

        register_rest_route( 'app-for-delivery-drivers/v1', '/set_track_me_status', array(
            array(
                'methods'             => 'POST',
                'callback'            => array( $this, 'set_track_me_status' ),
                'permission_callback' => array( $this, 'permissions_check' ),
                'args'                => array(
                    'driver_email' => array(
                        'required'          => true,
                        'type'              => 'string',
                        'description'       => 'Email address of the driver.',
                        'sanitize_callback' => 'sanitize_email',
                        'validate_callback' => array( $this, 'validate_email' ),
                    ),
                    'track_me_status' => array(
                        'required'          => true,
                        'type'              => 'number',
                        'description'       => 'Tracking status.',
                        'sanitize_callback' => array( $this, 'sanitize_float' ),
                        'validate_callback' => array( $this, 'validate_latitude' ),
                    ),
                   
                ),
            ),
        ) );

    }

    /**
 * Sanitizes a value to float.
 *
 * @param mixed           $value   The parameter value.
 * @param WP_REST_Request $request The current request.
 * @param string          $param   The parameter name.
 * @return float|null The sanitized float value or null if invalid.
 */
public function sanitize_float( $value, $request, $param ) {
    if ( is_numeric( $value ) ) {
        return floatval( $value );
    }
    return null;
}

    /**
     * Permission callback to ensure the request is authorized.
     *
     * @param WP_REST_Request $request The REST request.
     * @return bool|WP_Error True if the request has access, WP_Error otherwise.
     */
    public function permissions_check( $request ) {
        // Retrieve the Authorization header
        $auth_header = $request->get_header( 'Authorization' );
    
        if ( ! $auth_header ) {
            return new WP_Error( 'forbidden', __( 'No Authorization header present.' ), array( 'status' => 403 ) );
        }
    
        // Check if the Authorization header is in the Bearer token format
        if ( ! preg_match( '/Bearer\s(\S+)/', $auth_header, $matches ) ) {
            return new WP_Error( 'forbidden', __( 'Invalid Authorization header format.' ), array( 'status' => 403 ) );
        }
    
        // Extract the token from the header
        $token = $matches[1];
    
        // Retrieve the valid token from the WordPress options
        $valid_token = get_option( 'afdd_app_token' );
    
        // Validate the token using a timing-attack safe comparison
        if ( $valid_token && hash_equals( $valid_token, $token ) ) {
            return true;
        }
    
        return new WP_Error( 'forbidden', __( 'You do not have permissions to perform this action.' ), array( 'status' => 403 ) );
    }

    /**
     * Validates the email address.
     *
     * @param mixed           $value   The parameter value.
     * @param WP_REST_Request $request The current request.
     * @param string          $param   The parameter name.
     * @return bool True if valid, false otherwise.
     */
    public function validate_email( $value, $request, $param ) {
        return is_email( $value );
    }

    /**
     * Validates the latitude value.
     *
     * @param mixed           $value   The parameter value.
     * @param WP_REST_Request $request The current request.
     * @param string          $param   The parameter name.
     * @return bool True if valid, false otherwise.
     */
    public function validate_latitude( $value, $request, $param ) {
        return is_numeric( $value ) && $value >= -90 && $value <= 90;
    }

    /**
     * Validates the longitude value.
     *
     * @param mixed           $value   The parameter value.
     * @param WP_REST_Request $request The current request.
     * @param string          $param   The parameter name.
     * @return bool True if valid, false otherwise.
     */
    public function validate_longitude( $value, $request, $param ) {
        return is_numeric( $value ) && $value >= -180 && $value <= 180;
    }


     /**
     * Callback function to set the driver's location.
     *
     * @param WP_REST_Request $request The REST request.
     * @return WP_REST_Response|WP_Error The response or error.
     */
    public function set_track_me_status( $request ) {

        // Retrieve JSON parameters
        $params = $request->get_json_params();
  
        $driver_email    = isset($params['driver_email']) ? sanitize_email($params['driver_email']) : null;
        $track_me_status = isset($params['track_me_status']) ? floatval($params['track_me_status']) : null;
        

        if ( empty($driver_email) || is_null($track_me_status) ) {
            error_log("Missing required parameters.");
            return new WP_Error( 'missing_params', __( 'Missing required parameters.' ), array( 'status' => 400 ) );
        }

        // Get the user by email.
        $user = get_user_by( 'email', $driver_email );

        if ( ! $user ) {
            return new WP_Error( 'user_not_found', __( 'No user found with the provided email address.' ), array( 'status' => 404 ) );
        }

        // Check if the user has the 'driver' role.
        if ( ! in_array( 'driver', (array) $user->roles, true ) ) {
            return new WP_Error( 'invalid_role', __( 'The user does not have the driver role.' ), array( 'status' => 403 ) );
        }

        $driver_id = $user->ID;

        // Ensure the LDDFW_Tracking class is available.
        if ( ! class_exists( 'LDDFW_Tracking' ) ) {
            return new WP_Error( 'tracking_class_missing', __( 'Tracking class not found.' ), array( 'status' => 500 ) );
        }

        $tracking = new LDDFW_Tracking();

        // Attempt to set the driver's tracking position.
        $result = $tracking->lddfw_driver_tracking_status( $driver_id, $track_me_status );
        
        if ( $result ) {
            return new WP_REST_Response( array(
                'success' => true,
                'message' => __( 'Tracking status updated successfully.' ),
            ), 200 );
        } else {
            return new WP_Error( 'update_failed', __( 'Failed to update tracking status.' ), array( 'status' => 500 ) );
        }
    }


    /**
     * Callback function to set the driver's location.
     *
     * @param WP_REST_Request $request The REST request.
     * @return WP_REST_Response|WP_Error The response or error.
     */
    public function set_driver_location( $request ) {

        // Retrieve JSON parameters
        $params = $request->get_json_params();
  
        $driver_email = isset($params['driver_email']) ? sanitize_email($params['driver_email']) : null;
        $latitude     = isset($params['latitude']) ? floatval($params['latitude']) : null;
        $longitude    = isset($params['longitude']) ? floatval($params['longitude']) : null;
        $speed        = isset($params['speed']) ? floatval($params['speed']) : null;

        if ( empty($driver_email) || is_null($latitude) || is_null($longitude) ) {
            error_log("Missing required parameters.");
            return new WP_Error( 'missing_params', __( 'Missing required parameters.' ), array( 'status' => 400 ) );
        }

        // Get the user by email.
        $user = get_user_by( 'email', $driver_email );

        if ( ! $user ) {
            return new WP_Error( 'user_not_found', __( 'No user found with the provided email address.' ), array( 'status' => 404 ) );
        }

        // Check if the user has the 'driver' role.
        if ( ! in_array( 'driver', (array) $user->roles, true ) ) {
            return new WP_Error( 'invalid_role', __( 'The user does not have the driver role.' ), array( 'status' => 403 ) );
        }

        $driver_id = $user->ID;

        // Ensure the LDDFW_Tracking class is available.
        if ( ! class_exists( 'LDDFW_Tracking' ) ) {
            return new WP_Error( 'tracking_class_missing', __( 'Tracking class not found.' ), array( 'status' => 500 ) );
        }

        $tracking = new LDDFW_Tracking();

        // Attempt to set the driver's tracking position.
        $result = $tracking->lddfw_set_driver_tracking_position( $driver_id, $latitude, $longitude, $speed );

        if ( $result ) {
            return new WP_REST_Response( array(
                'success' => true,
                'message' => __( 'Driver location updated successfully.' ),
            ), 200 );
        } else {
            return new WP_Error( 'update_failed', __( 'Failed to update driver location.' ), array( 'status' => 500 ) );
        }
    }
}

// Initialize the API class.
new App_For_Delivery_Drivers_API();
